@@ -267,7 +267,7 @@ void cdi_hook_ldconfig(struct container *c, void *d)
             (verbose >= LL_TRACE ? "-v" : NULL), NULL);
 }
 
-/** Installl a guest hook that runs @c ldconfig(8) on the directories the
+/** Install a guest hook that runs @c ldconfig(8) on the directories the
     requested CDI specs wanted. If there are none, do nothing, i.e., install
     no hook rather than a hook that processes no directories.
 
@@ -353,8 +353,9 @@ void cdi_init(struct cdi_config *cf)
    T__ (cdi_specs == NULL);
    cdi_specs = list_new(0, sizeof(struct cdi_spec));
 
-   // Read CDI spec files specifically requested. This loop will error out if
-   // any of them are malformed or not found.
+   // Read CDI spec files requested by path. This loop will error out if any
+   // files are malformed or not found, i.e., when this loop is done, we know
+   // any files given with --cdi are present and valid.
    for (int i = 0; cf->ids[i] != NULL; i++)
       if (cdi_resid_path_p(cf->ids[i])) {
          struct cdi_spec *spec = cdi_read_maybe(cdi_specs, cf->ids[i]);
@@ -363,14 +364,19 @@ void cdi_init(struct cdi_config *cf)
       } else
          req_by_kind = true;
 
-   // Read CDI spec files in configured directories if neccessary, and keep
-   // them if requested.
-   if (cf->res_all_p || req_by_kind)
+   // If the user requested all CDI (--cdi with no argument) or CDI by kind
+   // (--cdi=foo/bar), read all spec files we can find. If we get past this
+   // block, we found at least one spec file containing what we wanted.
+   if (cf->res_all_p || req_by_kind) {
+      size_t dir_ct = 0;
+      size_t file_ct = 0;
       for (int i = 0; cf->spec_dirs[i] != NULL; i++) {
          char **entries;
-         VERBOSE("CDI: looking for spec files in: %s", cf->spec_dirs[i]);
-         if (!path_exists(cf->spec_dirs[i], NULL, true)) {
-            FATAL(-1, "CDI: spec dir not found, skipping: %s", cf->spec_dirs[i]);
+         if (path_exists(cf->spec_dirs[i], NULL, true)) {
+            VERBOSE("CDI: looking for resource files in: %s", cf->spec_dirs[i]);
+            dir_ct++;
+         } else {
+            VERBOSE("CDI: directory not found, skipping: %s", cf->spec_dirs[i]);
             continue;
          }
          entries = dir_glob(cf->spec_dirs[i], "*.json");
@@ -378,16 +384,32 @@ void cdi_init(struct cdi_config *cf)
             struct cdi_spec *spec;
             spec = cdi_read_maybe(cdi_specs,
                                   path_join(cf->spec_dirs[i], entries[j]));
-            if (spec != NULL && cdi_requested(cf, spec))
-               list_append((void **)&cdi_specs, spec, sizeof(*spec));
+            if (spec != NULL) {
+               file_ct++;
+               if (cdi_requested(cf, spec))
+                  list_append((void **)&cdi_specs, spec, sizeof(*spec));
+            }
          }
       }
+      if (dir_ct == 0)
+         FATAL(-1, "CDI: no directories found: %s",
+               list_stringify(cf->spec_dirs, ':'));
+      if (file_ct == 0)
+         FATAL(-1, "CDI: no resource files found: %s",
+                   list_stringify(cf->spec_dirs, ':'));
+   }
 
    // Log parsed CDI specs and validate count.
    spec_ct = list_count(cdi_specs, sizeof(cdi_specs[0]));
    DEBUG("CDI: read %zu specs", spec_ct);
-   if (spec_ct == 0 && (cf->res_all_p || cf->ids[0] != NULL))
-      FATAL(-1, "CDI: no specification files found");
+   if (spec_ct == 0) {
+      if (req_by_kind)
+         FATAL(-1, "CDI: no matching resources found");
+      else {
+         // Nothing to do, and we did nothing.
+         T__ (cf->ids[0] == NULL && !cf->res_all_p);
+      }
+   }
    for (size_t i = 0; i < spec_ct; i++)
       cdi_log(&cdi_specs[i]);
 }
@@ -478,7 +500,7 @@ struct cdi_spec *cdi_read_maybe(struct cdi_spec *specs, const char *path)
    // Don’t read file if we already did. It’s relatively easy to give a spec
    // file more than once, e.g. if it’s in the search path and also an
    // argument to --device.
-   Zfe (stat(path, &st), "can’t stat CDI spec: %s", path);
+   Zfe (stat(path, &st), "can't stat CDI resource file: %s", path);
    for (int i = 0; specs[i].kind != NULL; i++) {
       if (st.st_dev == specs[i].src_dev && st.st_ino == specs[i].src_ino) {
          VERBOSE("CDI: spec already read, skipping: %s", path);
